@@ -1,22 +1,37 @@
-"""
-testfixtures.comparison
------------------------
-"""
-
+from collections import OrderedDict
 from decimal import Decimal
 from difflib import unified_diff
-from functools import partial
+from functools import partial as partial_type, partial, reduce
+from operator import __or__
 from pprint import pformat
-from re import compile, MULTILINE
 from types import GeneratorType
+import re
 
 from testfixtures import not_there
-from testfixtures.compat import ClassType, Iterable, Unicode, basestring, PY3
+from testfixtures.compat import ClassType, Iterable, Unicode, basestring, PY3, PY2
 from testfixtures.resolve import resolve
 from testfixtures.utils import indent
 from testfixtures.mock import parent_name, mock_call, unittest_mock_call
 
 
+def diff(x, y, x_label='', y_label=''):
+    """
+    A shorthand function that uses :mod:`difflib` to return a
+    string representing the differences between the two string
+    arguments.
+
+    Most useful when comparing multi-line strings.
+    """
+    return '\n'.join(
+        unified_diff(
+            x.split('\n'),
+            y.split('\n'),
+            x_label or 'first',
+            y_label or 'second',
+            lineterm='')
+    )
+
+
 def compare_simple(x, y, context):
     """
     Returns a very simple textual difference between the two supplied objects.
@@ -29,13 +44,15 @@ def compare_simple(x, y, context):
                 return compare_with_type(x, y, context)
             x_attrs = _extract_attrs(x)
             y_attrs = _extract_attrs(y)
-            diff = _compare_mapping(x_attrs, y_attrs, context, x,
-                                    'attributes ', '.%s')
-            if not diff:
-                return 'Both %s and %s appear as %r, but are not equal!' % (
-                    context.x_label or 'x', context.y_label or 'y', repr_x
-                )
-            return diff
+            diff_ = None
+            if not (x_attrs is None or y_attrs is None):
+                diff_ = _compare_mapping(x_attrs, y_attrs, context, x,
+                                         'attributes ', '.%s')
+            if diff_:
+                return diff_
+            return 'Both %s and %s appear as %r, but are not equal!' % (
+                context.x_label or 'x', context.y_label or 'y', repr_x
+            )
         return context.label('x', repr_x) + ' != ' + context.label('y', repr_y)
 
 
@@ -100,7 +117,7 @@ def compare_object(x, y, context, ignore_attributes=()):
     y_attrs = _extract_attrs(y, _attrs_to_ignore(context, ignore_attributes, y))
     if x_attrs is None or y_attrs is None or not (x_attrs and y_attrs):
         return compare_simple(x, y, context)
-    if x_attrs != y_attrs:
+    if context.ignore_eq or x_attrs != y_attrs:
         return _compare_mapping(x_attrs, y_attrs, context, x,
                                 'attributes ', '.%s')
 
@@ -131,7 +148,7 @@ def compare_with_type(x, y, context):
     return '{x} != {y}'.format(**to_render)
 
 
-def compare_sequence(x, y, context):
+def compare_sequence(x, y, context, prefix=True):
     """
     Returns a textual description of the differences between the two
     supplied sequences.
@@ -147,7 +164,7 @@ def compare_sequence(x, y, context):
     if l_x == l_y and i == l_x:
         return
 
-    return ('sequence not as expected:\n\n'
+    return (('sequence not as expected:\n\n' if prefix else '')+
             'same:\n%s\n\n'
             '%s:\n%s\n\n'
             '%s:\n%s') % (pformat(x[:i]),
@@ -291,7 +308,8 @@ def compare_set(x, y, context):
             ))
     return '\n'.join(lines)+'\n'
 
-trailing_whitespace_re = compile('\s+$', MULTILINE)
+
+trailing_whitespace_re = re.compile(r'\s+$', re.MULTILINE)
 
 
 def strip_blank_lines(text):
@@ -425,7 +443,7 @@ _registry = {
     mock_call.__class__: compare_call,
     unittest_mock_call.__class__: compare_call,
     BaseException: compare_exception,
-    partial: compare_partial,
+    partial_type: compare_partial,
     }
 
 if PY3:
@@ -552,8 +570,9 @@ class CompareContext(object):
         return '\n\nWhile comparing %s: ' % ''.join(self.breadcrumbs[1:])
 
     def seen(self, x, y):
-        # don't get confused by string interning:
-        if isinstance(x, basestring) and isinstance(y, basestring):
+        # don't get confused by interning:
+        singleton_types = basestring, int, float
+        if isinstance(x, singleton_types) and isinstance(y, singleton_types):
             return False
         key = id(x), id(y)
         if key in self._seen:
@@ -604,6 +623,10 @@ class CompareContext(object):
             self.breadcrumbs.pop()
 
 
+def _resolve_lazy(source):
+    return str(source() if callable(source) else source)
+
+
 def compare(*args, **kw):
     """
     Compare two objects, raising an :class:`AssertionError` if they are not
@@ -621,11 +644,13 @@ def compare(*args, **kw):
 
     :param prefix: If provided, in the event of an :class:`AssertionError`
                    being raised, the prefix supplied will be prepended to the
-                   message in the :class:`AssertionError`.
+                   message in the :class:`AssertionError`. This may be a
+                   callable, in which case it will only be resolved if needed.
 
     :param suffix: If provided, in the event of an :class:`AssertionError`
                    being raised, the suffix supplied will be appended to the
-                   message in the :class:`AssertionError`.
+                   message in the :class:`AssertionError`. This may be a
+                   callable, in which case it will only be resolved if needed.
 
     :param x_label: If provided, in the event of an :class:`AssertionError`
                     being raised, the object passed as the first positional
@@ -633,7 +658,7 @@ def compare(*args, **kw):
                     with this string in the message in the
                     :class:`AssertionError`.
 
-    :param x_label: If provided, in the event of an :class:`AssertionError`
+    :param y_label: If provided, in the event of an :class:`AssertionError`
                     being raised, the object passed as the second positional
                     argument, or ``y`` keyword argument, will be labelled
                     with this string in the message in the
@@ -676,41 +701,78 @@ def compare(*args, **kw):
 
     message = context.message
     if prefix:
-        message = prefix + ': ' + message
+        message = _resolve_lazy(prefix) + ': ' + message
     if suffix:
-        message += '\n' + suffix
+        message += '\n' + _resolve_lazy(suffix)
 
     if raises:
         raise AssertionError(message)
     return message
 
 
-class Comparison(object):
+class StatefulComparison(object):
     """
-    These are used when you need to compare objects
-    that do not natively support comparison.
+    A base class for stateful comparison objects.
+    """
+
+    failed = ''
+    expected = None
+    name_attrs = ()
+
+    def __eq__(self, other):
+        return not(self != other)
+
+    def name(self):
+        name = type(self).__name__
+        if self.name_attrs:
+            name += '(%s)' % ', '.join('%s=%r' % (n, getattr(self, n)) for n in self.name_attrs)
+        return name
+
+    def body(self):
+        return pformat(self.expected)[1:-1]
+
+    def __repr__(self):
+        name = self.name()
+        body = self.failed or self.body()
+        prefix = '<%s%s>' % (name, self.failed and '(failed)' or '')
+        if '\n' in body:
+            return '\n'+prefix+'\n'+body.strip('\n')+'\n'+'</%s>' % name
+        elif body:
+            return prefix + body + '</>'
+        return prefix
+
+
+class Comparison(StatefulComparison):
+    """
+    These are used when you need to compare an object's type, a subset of its attributes
+    or make equality checks with objects that do not natively support comparison.
 
     :param object_or_type: The object or class from which to create the
                            :class:`Comparison`.
 
-    :param attribute_dict: An optional dictionary containing attibutes
+    :param attribute_dict: An optional dictionary containing attributes
                            to place on the :class:`Comparison`.
 
-    :param strict: If true, any expected attributes not present or extra
-                   attributes not expected on the object involved in the
-                   comparison will cause the comparison to fail.
+    :param partial:
+      If true, only the specified attributes will be checked and any extra attributes
+      of the object being compared with will be ignored.
 
     :param attributes: Any other keyword parameters passed will placed
                        as attributes on the :class:`Comparison`.
-    """
 
-    failed = None
+    :param strict:
+
+      .. deprecated:: 6.16.0
+
+        Use ``partial`` instead.
+    """
 
     def __init__(self,
                  object_or_type,
                  attribute_dict=None,
-                 strict=True,
+                 partial=False,
                  **attributes):
+        self.partial = partial or not attributes.pop('strict', True)
         if attributes:
             if attribute_dict is None:
                 attribute_dict = attributes
@@ -728,77 +790,306 @@ class Comparison(object):
             c = object_or_type.__class__
             if attribute_dict is None:
                 attribute_dict = _extract_attrs(object_or_type)
-        self.c = c
-        self.v = attribute_dict
-        self.strict = strict
+        self.expected_type = c
+        self.expected_attributes = attribute_dict
 
-    def __eq__(self, other):
-        if self.c is not other.__class__:
+    def __ne__(self, other):
+        # .__class__ is important for Py2 compatibility.
+        if self.expected_type is not other.__class__:
             self.failed = 'wrong type'
-            return False
-
-        if self.v is None:
             return True
 
-        remaining_keys = set(self.v.keys())
-        if self.strict:
-            v = _extract_attrs(other)
-            remaining_keys -= set(v.keys())
-        else:
-            v = {}
+        if self.expected_attributes is None:
+            return False
 
-        while remaining_keys:
-            k = remaining_keys.pop()
+        attribute_names = set(self.expected_attributes.keys())
+        if self.partial:
+            actual_attributes = {}
+        else:
+            actual_attributes = _extract_attrs(other)
+            attribute_names -= set(actual_attributes)
+
+        for name in attribute_names:
             try:
-                v[k] = getattr(other, k)
+                actual_attributes[name] = getattr(other, name)
             except AttributeError:
                 pass
 
         kw = {'x_label': 'Comparison', 'y_label': 'actual'}
         context = CompareContext(kw)
-        self.failed = _compare_mapping(self.v,
-                                       v,
+        self.failed = _compare_mapping(self.expected_attributes,
+                                       actual_attributes,
                                        context,
                                        obj_for_class=not_there,
                                        prefix='attributes ',
                                        breadcrumb='.%s',
-                                       check_y_not_x=self.strict)
-        return not self.failed
+                                       check_y_not_x=not self.partial)
+        return bool(self.failed)
 
-    def __ne__(self, other):
-        return not(self == other)
+    def name(self):
+        name = 'C:'
+        module = getattr(self.expected_type, '__module__', None)
+        if module:
+            name = name + module + '.'
+        name += (getattr(self.expected_type, '__name__', None) or repr(self.expected_type))
+        return name
 
-    def __repr__(self):
-        name = getattr(self.c, '__module__', '')
-        if name:
-            name += '.'
-        name += getattr(self.c, '__name__', '')
-        if not name:
-            name = repr(self.c)
-
-        text = ''
-        if self.failed:
-            text = self.failed
-        elif self.v:
+    def body(self):
+        if self.expected_attributes:
+            # if we're not failed, show what we will expect:
             lines = []
-            for k, v in sorted(self.v.items()):
+            for k, v in sorted(self.expected_attributes.items()):
                 rv = repr(v)
                 if '\n' in rv:
                     rv = indent(rv)
                 lines.append('%s: %s' % (k, rv))
-            text = '\n'.join(lines)
-            if len(lines) > 1:
-                text = '\n'+text
+            return '\n'.join(lines)
+        return ''
 
-        r = '<C%s:%s>' % (self.failed and '(failed)' or '', name)
-        if '\n' in text:
-            r = '\n'+r+text+'\n'
+
+class SequenceComparison(StatefulComparison):
+    """
+    An object that can be used in comparisons of expected and actual
+    sequences.
+
+    :param expected: The items expected to be in the sequence.
+    :param ordered:
+      If the items are expected to be in the order specified.
+      Defaults to ``True``.
+    :param partial:
+      If any items not expected should be ignored.
+      Defaults to ``False``.
+    :param recursive:
+      If a difference is found, recursively compare the item where
+      the difference was found to highlight exactly what was different.
+      Defaults to ``False``.
+    """
+
+    name_attrs = ('ordered', 'partial')
+
+    def __init__(self, *expected, **kw):
+        self.expected = expected
+        # py2 :-(
+        self.ordered = kw.pop('ordered', True)
+        self.partial = kw.pop('partial', False)
+        self.recursive = kw.pop('recursive', False)
+        assert not kw, 'unexpected parameter'
+        self.checked_indices = set()
+
+    def __ne__(self, other):
+        try:
+            actual = original_actual = list(other)
+        except TypeError:
+            self.failed = 'bad type'
+            return True
+        expected = list(self.expected)
+        actual = list(actual)
+
+        matched = []
+        matched_expected_indices = []
+        matched_actual_indices = []
+
+        missing_from_expected = actual
+        missing_from_expected_indices = actual_indices = list(range(len(actual)))
+
+        missing_from_actual = []
+        missing_from_actual_indices = []
+
+        start = 0
+        for e_i, e in enumerate(expected):
+            try:
+                i = actual.index(e, start)
+                a_i = actual_indices.pop(i)
+            except ValueError:
+                missing_from_actual.append(e)
+                missing_from_actual_indices.append(e_i)
+            else:
+                matched.append(missing_from_expected.pop(i))
+                matched_expected_indices.append(e_i)
+                matched_actual_indices.append(a_i)
+                self.checked_indices.add(a_i)
+                if self.ordered:
+                    start = i
+
+        matches_in_order = matched_actual_indices == sorted(matched_actual_indices)
+        all_matched = not (missing_from_actual or missing_from_expected)
+        partial_match = self.partial and not missing_from_actual
+
+        if (matches_in_order or not self.ordered) and (all_matched or partial_match):
+            return False
+
+        expected_indices = matched_expected_indices+missing_from_actual_indices
+        actual_indices = matched_actual_indices
+
+        if self.partial:
+            # try to give a clue as to what didn't match:
+            if self.recursive and self.ordered and missing_from_expected:
+                actual_indices.append(missing_from_expected_indices.pop(0))
+                missing_from_expected.pop(0)
+
+            ignored = missing_from_expected
+            missing_from_expected = None
         else:
-            r += text
-        if text:
-            r += '</C>'
+            actual_indices += missing_from_expected_indices
+            ignored = None
 
-        return r
+        message = []
+
+        def add_section(name, content):
+            if content:
+                message.append(name+':\n'+pformat(content))
+
+        add_section('ignored', ignored)
+
+        if self.ordered:
+            message.append(compare(
+                expected=[self.expected[i] for i in sorted(expected_indices)],
+                actual=[original_actual[i] for i in sorted(actual_indices)],
+                recursive=self.recursive,
+                raises=False
+            ).split('\n\n', 1)[1])
+        else:
+            add_section('same', matched)
+            add_section('in expected but not actual', missing_from_actual)
+            add_section('in actual but not expected', missing_from_expected)
+
+        self.failed = '\n\n'.join(message)
+        return True
+
+
+class Subset(SequenceComparison):
+    """
+    A shortcut for :class:`SequenceComparison` that checks if the
+    specified items are present in the sequence.
+    """
+
+    name_attrs = ()
+
+    def __init__(self, *expected):
+        super(Subset, self).__init__(*expected, ordered=False, partial=True)
+
+
+class Permutation(SequenceComparison):
+    """
+    A shortcut for :class:`SequenceComparison` that checks if the set of items
+    in the sequence is as expected, but without checking ordering.
+    """
+
+    def __init__(self, *expected):
+        super(Permutation, self).__init__(*expected, ordered=False, partial=False)
+
+
+class MappingComparison(StatefulComparison):
+    """
+    An object that can be used in comparisons of expected and actual
+    mappings.
+
+    :param expected_mapping:
+      The mapping that should be matched expressed as either a sequence of
+      ``(key, value)`` tuples or a mapping.
+    :param expected_items: The items that should be matched.
+    :param ordered:
+      If the keys in the mapping are expected to be in the order specified.
+      Defaults to ``False``.
+    :param partial:
+      If any keys not expected should be ignored.
+      Defaults to ``False``.
+    :param recursive:
+      If a difference is found, recursively compare the value where
+      the difference was found to highlight exactly what was different.
+      Defaults to ``False``.
+    """
+
+    name_attrs = ('ordered', 'partial')
+
+    def __init__(self, *expected_mapping, **expected_items):
+        # py2 :-(
+        self.ordered = expected_items.pop('ordered', False)
+        self.partial = expected_items.pop('partial', False)
+        self.recursive = expected_items.pop('recursive', False)
+
+        if PY2 and self.ordered:
+            if expected_items:
+                raise TypeError('order undefined on Python 2')
+            elif expected_mapping and type(expected_mapping[0]) is dict:
+                raise TypeError('dict order undefined on Python 2')
+
+        if len(expected_mapping) == 1:
+            expected = OrderedDict(*expected_mapping)
+        else:
+            expected = OrderedDict(expected_mapping)
+        expected.update(expected_items)
+
+        self.expected = expected
+
+    def body(self):
+        # this can all go away and use the super class once py2 is gone :'(
+        parts = []
+        text_length = 0
+        for key, value in self.expected.items():
+            part = repr(key)+': '+pformat(value)
+            text_length += len(part)
+            parts.append(part)
+        if text_length > 60:
+            sep = ',\n'
+        else:
+            sep = ', '
+        return sep.join(parts)
+
+    def __ne__(self, other):
+        try:
+            actual_keys = other.keys()
+            actual_mapping = dict(other.items())
+        except AttributeError:
+            self.failed = 'bad type'
+            return True
+
+        expected_keys = self.expected.keys()
+        expected_mapping = self.expected
+
+        if self.partial:
+            ignored_keys = set(actual_keys) - set(expected_keys)
+            for key in ignored_keys:
+                del actual_mapping[key]
+            # preserve the order:
+            actual_keys = [k for k in actual_keys if k not in ignored_keys]
+        else:
+            ignored_keys = None
+
+        mapping_differences = compare(
+            expected=expected_mapping,
+            actual=actual_mapping,
+            recursive=self.recursive,
+            raises=False
+        )
+
+        if self.ordered:
+            key_differences = compare(
+                expected=list(expected_keys),
+                actual=list(actual_keys),
+                recursive=self.recursive,
+                raises=False
+            )
+        else:
+            key_differences = None
+
+        if key_differences or mapping_differences:
+
+            message = []
+
+            if ignored_keys:
+                message.append('ignored:\n'+pformat(sorted(ignored_keys)))
+
+            if mapping_differences:
+                message.append(mapping_differences.split('\n\n', 1)[1])
+
+            if key_differences:
+                message.append('wrong key order:\n\n'+key_differences.split('\n\n', 1)[1])
+
+            self.failed = '\n\n'.join(message)
+
+            return True
+        return False
 
 
 class StringComparison:
@@ -812,9 +1103,21 @@ class StringComparison:
                          :class:`StringComparison` is compared with
                          any :class:`basestring` instance.
 
+    :param flags: Flags passed to :func:`re.compile`.
+
+    :param flag_names: See the :ref:`examples <stringcomparison>`.
     """
-    def __init__(self, regex_source):
-        self.re = compile(regex_source)
+    def __init__(self, regex_source, flags=None, **flag_names):
+        args = [regex_source]
+
+        flags_ = []
+        if flags:
+            flags_.append(flags)
+        flags_.extend(getattr(re, f.upper()) for f in flag_names)
+        if flags_:
+            args.append(reduce(__or__, flags_))
+
+        self.re = re.compile(*args)
 
     def __eq__(self, other):
         if not isinstance(other, basestring):
@@ -863,24 +1166,6 @@ class RoundComparison:
         return '<R:%s to %i digits>' % (self.rounded, self.precision)
 
 
-def diff(x, y, x_label='', y_label=''):
-    """
-    A shorthand function that uses :mod:`difflib` to return a
-    string representing the differences between the two string
-    arguments.
-
-    Most useful when comparing multi-line strings.
-    """
-    return '\n'.join(
-        unified_diff(
-            x.split('\n'),
-            y.split('\n'),
-            x_label or 'first',
-            y_label or 'second',
-            lineterm='')
-    )
-
-
 class RangeComparison:
     """
     An object that can be used in comparisons of orderable types to
