@@ -1,3 +1,4 @@
+import re
 from datetime import date, datetime
 from decimal import Decimal
 
@@ -38,6 +39,15 @@ marker = object()
 _compare = compare
 
 
+class Lazy:
+
+    def __init__(self, message):
+        self.message = message
+
+    def __str__(self):
+        return self.message
+
+
 def check_raises(x=marker, y=marker, message=None, regex=None,
                  compare=compare, **kw):
     args = []
@@ -235,14 +245,14 @@ class TestCompare(CompareHelper, TestCase):
         self.check_raises(
             C(e1), e2,
             ("\n"
-             "<C(failed):{}.ValueError>\n"
+             "<C:{module}.ValueError(failed)>\n"
              "attributes differ:\n"
              "'args': ('some message',) (Comparison) "
              "!= ('some other message',) (actual)\n"
-             "</C>"
-             " != ValueError('some other message'{})"
-             ).format(exception_module,
-                      '' if PY_37_PLUS else ','))
+             "</C:{module}.ValueError>"
+             " != ValueError('some other message'{message})"
+             ).format(module=exception_module,
+                      message='' if PY_37_PLUS else ','))
 
     def test_sequence_long(self):
         self.check_raises(
@@ -390,6 +400,26 @@ class TestCompare(CompareHelper, TestCase):
             "'x': 1 != 2"
             )
 
+    def test_dict_identical_non_matching_ints(self):
+        self.check_raises(
+            dict(x=1, y=1), dict(x=2, y=2),
+            "dict not as expected:\n"
+            "\n"
+            "values differ:\n"
+            "'x': 1 != 2\n"
+            "'y': 1 != 2"
+            )
+
+    def test_dict_identical_non_matching_floats(self):
+        self.check_raises(
+            dict(x=1.0, y=1.0), dict(x=2.0, y=2.0),
+            "dict not as expected:\n"
+            "\n"
+            "values differ:\n"
+            "'x': 1.0 != 2.0\n"
+            "'y': 1.0 != 2.0"
+            )
+
     def test_dict_labels_specified(self):
         self.check_raises(
             dict(x=1, y=2), dict(x=2, z=3),
@@ -1036,6 +1066,17 @@ b
             prefix='file content'
             )
 
+    def test_prefix_callable(self):
+        with ShouldAssert('foo: 1 != 2'):
+            compare(1, 2, prefix=lambda: 'foo')
+
+    def test_prefix_stringable(self):
+        with ShouldAssert('foo: 1 != 2'):
+            compare(1, 2, prefix=Lazy('foo'))
+
+    def test_prefix_lazy(self):
+        compare(2, 2, prefix=Mock(side_effect=Exception('boom!')))
+
     def test_suffix(self):
         self.check_raises(
             1, 2,
@@ -1044,6 +1085,17 @@ b
             suffix='additional context',
             )
 
+    def test_suffix_callable(self):
+        with ShouldAssert('1 != 2\n3'):
+            compare(1, 2, suffix=lambda: 3)
+
+    def test_suffix_stringable(self):
+        with ShouldAssert('1 != 2\nfoo'):
+            compare(1, 2, suffix=Lazy('foo'))
+
+    def test_suffix_lazy(self):
+        compare(2, 2, suffix=Mock(side_effect=Exception('boom!')))
+
     def test_labels_multiline(self):
         self.check_raises(
             'x'*5+'\n'+'y'*5, 'x'*5+'\n'+'z'*5,
@@ -1508,6 +1560,25 @@ b
             ignore_eq=True
         )
 
+    def test_nested_django_orm_in_object(self):
+        class MyObject(object):
+            def __init__(self, orm):
+                self.orm = orm
+
+        self.check_raises(
+            message="MyObject not as expected:\n"
+                    "\n"
+                    "attributes differ:\n"
+                    "'orm': OrmObj: 1 (expected) != OrmObj: 2 (actual)\n"
+                    "\n"
+                    "While comparing .orm: OrmObj not as expected:\n"
+                    "\n"
+                    "attributes differ:\n"
+                    "'a': 1 (expected) != 2 (actual)",
+            expected=MyObject(self.OrmObj(1)),
+            actual=MyObject(self.OrmObj(2)),
+            ignore_eq=True)
+
     def test_mock_call_same(self):
         m = Mock()
         m.foo(1, 2, x=3)
@@ -1869,6 +1940,16 @@ b
             "While comparing [2]: 'foo' != 'bar'"
             )
 
+    def test_regex(self):
+        if PY2:
+            return
+        shared_prefix = "a" * 199
+        self.check_raises(
+            re.compile(shared_prefix + "x"),
+            re.compile(shared_prefix + "y"),
+            'Both x and y appear as "re.compile(\''+'a'*199+')", but are not equal!'
+            )
+
 
 class TestIgnore(CompareHelper):
 
