@@ -1,5 +1,5 @@
 from __future__ import print_function
-from logging import getLogger
+from logging import getLogger, ERROR, Filter
 from textwrap import dedent
 from unittest import TestCase
 from warnings import catch_warnings
@@ -7,7 +7,7 @@ from warnings import catch_warnings
 from testfixtures.shouldraise import ShouldAssert
 from testfixtures.mock import Mock
 
-from testfixtures import Replacer, LogCapture, compare
+from testfixtures import Replacer, LogCapture, compare, Replace
 
 root = getLogger()
 one = getLogger('one')
@@ -15,6 +15,11 @@ two = getLogger('two')
 child = getLogger('one.child')
 
 
+class DummyFilter(Filter):
+    def filter(self, _):
+        return True
+
+
 class TestLogCapture(TestCase):
 
     def test_simple(self):
@@ -25,6 +30,77 @@ class TestLogCapture(TestCase):
         root.info('after')
         assert str(l) == "root INFO\n  during"
 
+    def test_simple_strict(self):
+        log_capture = LogCapture(ensure_checks_above=ERROR)
+        root.error('during')
+        log_capture.uninstall()
+        with ShouldAssert("Not asserted ERROR log(s): [('root', 'ERROR', 'during')]"):
+            log_capture.ensure_checked()
+
+    def test_simple_strict_re_defaulted(self):
+        with Replace('testfixtures.LogCapture.default_ensure_checks_above', ERROR):
+            LogCapture.default_ensure_checks_above = ERROR
+            log_capture = LogCapture()
+            root.error('during')
+            log_capture.uninstall()
+            with ShouldAssert("Not asserted ERROR log(s): [('root', 'ERROR', 'during')]"):
+                log_capture.ensure_checked()
+
+    def test_simple_strict_asserted_by_check(self):
+        log_capture = LogCapture(ensure_checks_above=ERROR)
+        root.error('during')
+        log_capture.uninstall()
+        log_capture.check(("root", "ERROR", "during"))
+        log_capture.ensure_checked()
+
+    def test_simple_strict_asserted_by_check_present_ordered(self):
+        log_capture = LogCapture(ensure_checks_above=ERROR)
+        root.error('during')
+        log_capture.uninstall()
+        log_capture.check_present(("root", "ERROR", "during"))
+        log_capture.ensure_checked()
+
+    def test_simple_strict_asserted_by_check_present_unordered(self):
+        log_capture = LogCapture(ensure_checks_above=ERROR)
+        root.error('during')
+        log_capture.uninstall()
+        log_capture.check_present(("root", "ERROR", "during"), order_matters=False)
+        log_capture.ensure_checked()
+
+    def test_simple_strict_not_asserted_by_check_present(self):
+        log_capture = LogCapture(ensure_checks_above=ERROR)
+        root.error('before')
+        root.error('during')
+        log_capture.uninstall()
+        log_capture.check_present(("root", "ERROR", "during"))
+        with ShouldAssert("Not asserted ERROR log(s): [('root', 'ERROR', 'before')]"):
+            log_capture.ensure_checked()
+
+    def test_simple_strict_asserted_by_containment(self):
+        log_capture = LogCapture(ensure_checks_above=ERROR)
+        root.error('during')
+        log_capture.uninstall()
+        assert ("root", "ERROR", "during") in log_capture
+        assert ("root", "INFO", "during") not in log_capture
+        log_capture.ensure_checked()
+
+    def test_simple_strict_asserted_by_mark_all_checked(self):
+        log_capture = LogCapture(ensure_checks_above=ERROR)
+        root.error('during')
+        log_capture.uninstall()
+        log_capture.mark_all_checked()
+        log_capture.ensure_checked()
+
+    def test_simple_strict_ctx(self):
+        with ShouldAssert("Not asserted ERROR log(s): [('root', 'ERROR', 'during')]"):
+            with LogCapture(ensure_checks_above=ERROR):
+                root.error('during')
+
+    def test_simple_strict_asserted_ctx(self):
+        with LogCapture(ensure_checks_above=ERROR) as log_capture:
+            root.error('during')
+            log_capture.check(("root", "ERROR", "during"))
+
     def test_specific_logger(self):
         l = LogCapture('one')
         root.info('1')
@@ -64,6 +140,10 @@ class TestLogCapture(TestCase):
         root = getLogger()
         child = getLogger('child')
 
+        # Add a dummy filter so we can verify it is swapped out
+        # during the capture, and swapped back in after `uninstall`.
+        root.addFilter(DummyFilter())
+
         # Lets also record the handlers for these loggers before
         # we start the test:
 
@@ -75,6 +155,11 @@ class TestLogCapture(TestCase):
         old_root_level=root.level
         old_child_level=child.level
 
+        # Also record the filters:
+
+        old_root_filters = root.filters[:]
+        old_child_filters = child.filters[:]
+
         # Now the test:
 
         try:
@@ -88,6 +173,9 @@ class TestLogCapture(TestCase):
             assert root.level == 1
             assert child.level == 1
 
+            assert root.filters == []
+            assert child.filters == []
+
             child.info('2')
             assert str(l1) == (
                 "root INFO\n  1\n"
@@ -96,6 +184,11 @@ class TestLogCapture(TestCase):
             assert str(l2) == (
                 "child INFO\n  2"
             )
+
+            # Add a dummy filter to the child,
+            # which should be removed by `uninstall`.
+            child.addFilter(DummyFilter())
+
             l2.uninstall()
             l1.uninstall()
             assert root.level == 49
@@ -109,6 +202,10 @@ class TestLogCapture(TestCase):
         assert root.handlers == before_root
         assert child.handlers == before_child
 
+        # Also check the filters were restored:
+        assert root.filters == old_root_filters
+        assert child.filters == old_child_filters
+
     def test_uninstall_all(self):
         before_handlers_root = root.handlers[:]
         before_handlers_child = child.handlers[:]
@@ -284,16 +381,17 @@ class TestCheckPresent(object):
         with LogCapture() as log:
             root.error('junk')
         with ShouldAssert(dedent("""\
-                sequence not as expected:
+                ignored:
+                [('root', 'ERROR', 'junk')]
                 
                 same:
-                ()
+                []
                 
                 expected:
-                (('root', 'INFO', 'one'),)
+                [('root', 'INFO', 'one')]
                 
                 actual:
-                (('root', 'ERROR', 'junk'),)""")):
+                []""")):
             log.check_present(
                 ('root', 'INFO', 'one'),
             )
@@ -302,16 +400,14 @@ class TestCheckPresent(object):
         with LogCapture(recursive_check=True) as log:
             root.error('junk')
         with ShouldAssert(dedent("""\
-                sequence not as expected:
-                
                 same:
-                ()
+                []
                 
                 expected:
-                (('root', 'INFO', 'one'),)
+                [('root', 'INFO', 'one')]
                 
                 actual:
-                (('root', 'ERROR', 'junk'),)
+                [('root', 'ERROR', 'junk')]
                 
                 While comparing [0]: sequence not as expected:
                 
@@ -337,19 +433,17 @@ class TestCheckPresent(object):
             root.warning('two')
             root.error('j2')
         with ShouldAssert(dedent("""\
-                sequence not as expected:
+                ignored:
+                [('root', 'ERROR', 'j1'), ('root', 'ERROR', 'three'), ('root', 'ERROR', 'j2')]
                 
                 same:
-                (('root', 'INFO', 'one'),)
+                [('root', 'INFO', 'one'), ('root', 'WARNING', 'two')]
                 
                 expected:
-                (('root', 'WARNING', 'two'), ('root', 'ERROR', 'three'))
+                [('root', 'ERROR', 'three')]
                 
                 actual:
-                (('root', 'ERROR', 'j1'),
-                 ('root', 'ERROR', 'three'),
-                 ('root', 'WARNING', 'two'),
-                 ('root', 'ERROR', 'j2'))""")):
+                []""")):
             log.check_present(
                 ('root', 'INFO', 'one'),
                 ('root', 'WARNING', 'two'),
@@ -374,16 +468,11 @@ class TestCheckPresent(object):
         with LogCapture() as log:
             root.error('junk')
         with ShouldAssert(dedent("""\
-                entries not as expected:
+                ignored:
+                [('root', 'ERROR', 'junk')]
                 
-                expected and found:
-                []
-                
-                expected but not found:
-                [('root', 'INFO', 'one')]
-                
-                other entries:
-                [('root', 'ERROR', 'junk')]""")):
+                in expected but not actual:
+                [('root', 'INFO', 'one')]""")):
             log.check_present(
                 ('root', 'INFO', 'one'),
                 order_matters=False
@@ -406,16 +495,17 @@ class TestCheckPresent(object):
             root.error('junk')
             root.error('three')
         with ShouldAssert(dedent("""\
-                sequence not as expected:
+                ignored:
+                ['one', 'junk', 'three']
                 
                 same:
-                ()
+                []
                 
                 expected:
-                ('two',)
+                ['two']
                 
                 actual:
-                ('one', 'junk', 'three')""")):
+                []""")):
             log.check_present('two')
 
     def test_bad_params(self):
@@ -462,16 +552,14 @@ class TestCheckPresent(object):
             root.error('junk')
             root.info('one')
         with ShouldAssert(dedent("""\
-                entries not as expected:
+                ignored:
+                [('root', 'ERROR', 'junk'), ('root', 'ERROR', 'junk')]
                 
-                expected and found:
-                [('root', 'INFO', 'one'), ('root', 'WARNING', 'two'), ('root', 'INFO', 'one')]
+                same:
+                [('root', 'INFO', 'one'), ('root', 'INFO', 'one'), ('root', 'WARNING', 'two')]
                 
-                expected but not found:
-                [('root', 'WARNING', 'two')]
-                
-                other entries:
-                [('root', 'ERROR', 'junk'), ('root', 'ERROR', 'junk')]""")):
+                in expected but not actual:
+                [('root', 'WARNING', 'two')]""")):
             log.check_present(
                 ('root', 'INFO', 'one'),
                 ('root', 'INFO', 'one'),
@@ -504,18 +592,17 @@ class TestCheckPresent(object):
             root.warning('two')
             root.error('junk')
         with ShouldAssert(dedent("""\
-                sequence not as expected:
+                ignored:
+                [('root', 'ERROR', 'junk'), ('root', 'ERROR', 'junk')]
                 
                 same:
-                (('root', 'INFO', 'one'),)
+                [('root', 'INFO', 'one'), ('root', 'WARNING', 'two')]
                 
                 expected:
-                (('root', 'WARNING', 'two'), ('root', 'ERROR', 'three'))
+                [('root', 'ERROR', 'three')]
                 
                 actual:
-                (('root', 'ERROR', 'junk'),
-                 ('root', 'WARNING', 'two'),
-                 ('root', 'ERROR', 'junk'))""")):
+                []""")):
             log.check_present(
                 ('root', 'INFO', 'one'),
                 ('root', 'WARNING', 'two'),
@@ -529,16 +616,14 @@ class TestCheckPresent(object):
             root.error('three')
             root.error('junk')
         with ShouldAssert(dedent("""\
-                entries not as expected:
+                ignored:
+                [('root', 'ERROR', 'junk'), ('root', 'ERROR', 'junk')]
                 
-                expected and found:
-                [('root', 'INFO', 'one'), ('root', 'ERROR', 'three')]
+                same:
+                [('root', 'ERROR', 'three'), ('root', 'INFO', 'one')]
                 
-                expected but not found:
-                [('root', 'WARNING', 'two')]
-                
-                other entries:
-                [('root', 'ERROR', 'junk'), ('root', 'ERROR', 'junk')]""")):
+                in expected but not actual:
+                [('root', 'WARNING', 'two')]""")):
             log.check_present(
                 ('root', 'ERROR', 'three'),
                 ('root', 'INFO', 'one'),
